package com.adhi.adhij;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NoInitialContextException;
import javax.sql.DataSource;


/**
 * The Data Access Object.  This class connects to the data
 * server and executes queries. 
 * @author DKRISHNA
 *
 */
public abstract class DataAccessObject implements Serializable
{
	private Connection connection;
	

	/**
	 * close the database connection
	 * @throws SQLException when the connection cannot he closed
	 */
	public void closeConnection() throws SQLException 
	{
		if (connection != null)
		{
				connection.close();
				connection = null;

		}	
	}
	
	/**
	 * This is a clean up operation.  it closes all the database
	 * resources (result set, statement and connection)
	 * @param result the result set
	 * @param stmt the statement
	 * @param exception if there is an exception already happened
	 * @throws DataException when the clean up operation fails
	 */
	public void finallyClose(ResultSet result, Statement stmt,
				Exception exception) throws DataException
	{
		try
		{
			if(result != null) result.close();
			if (stmt != null) stmt.close();
			closeConnection();
		}
		catch(SQLException e)
		{
			exception = e;
		}
		if(exception != null)
		{
			throw new DataException(exception.getMessage());
		}

	}

	/**
	 * opens the database connection
	 * @throws Exception if the database connection cannot be opened
	 */
	public void openConnection() throws Exception 
	{
		if (connection == null)
		{
				// The newInstance() call is a work around for some
				// broken Java implementations

				Class.forName((String)ApplicationParam.getValue(
									ApplicationParam.JDBC_DRIVER)).newInstance();
				
				connection = DriverManager.getConnection(
					(String)ApplicationParam.getValue(ApplicationParam.CONNECTION_URL)+
					"?user="+
					(String)ApplicationParam.getValue(ApplicationParam.DB_USER)+
					"&password="+
					(String)ApplicationParam.getValue(ApplicationParam.DB_PASSWORD)
					);
		}
		
	}
	
	/**
	 * open a database connection using a jndi name of a data source
	 * @param dsJNDI the JNDI name of a data source
	 * @throws DataException when the connection cannot be opened
	 */
	public void openConnection(String dsJNDI) throws DataException 
	{
		if (connection == null)
		{
			
			try{
				Context ctx = new InitialContext();
				
				DataSource ds = (DataSource)ctx.lookup( dsJNDI /*(String)
					ApplicationParam.getValue(ApplicationParam.DATA_SOURCE)*//*"java:comp/env/jdbc/spmDS"*/);
		
				if (ds != null) {
					connection = ds.getConnection();
				}
			}
			catch(NoInitialContextException e)
			{
				try {
					openConnection();
				} catch (Exception e1) {
					throw new DataException(e.getMessage());
				}
			}
			catch(Throwable e) {
				e.printStackTrace();
				throw new DataException(e.getMessage());
			}
		}
	}
	
	/**
	 * create a row set out of a result set
	 * @param rs the result set
	 * @return the rowset
	 * @throws SQLException if the rowset cannot be created
	 */
	public RowSet createRowSet(ResultSet rs) throws SQLException
	{
		ResultSetMetaData rsMeta = null;
		ArrayList row = null;
		RowSet rowSet = new RowSet();
		if(rs != null)
		{
			rsMeta = rs.getMetaData();
			int size = 0;
			int i=0;
			if(rsMeta != null && (size = rsMeta.getColumnCount()) > 0)
			{
				while (rs.next())
				{
					row = new ArrayList(size);
					for(i=0; i<size; i++)
					{
						addFieldToRow(row, rsMeta.getColumnType(i+1), rs, i);
					}
					rowSet.add(row);
				}
			}
		}
		return rowSet;
	}
	
	/**
	 * create a prepared statement using the connection
	 * @param query the query string
	 * @return the prepared statement
	 * @throws SQLException when the prepared statement cannot be created
	 */
	public PreparedStatement getPreparedStmt(String query) throws SQLException{
		PreparedStatement stmt = null;
	
			stmt = connection.prepareStatement(query);
		return stmt;
		
		
	}
	
	/**
	 * create a statement
	 * @return the statement
	 * @throws SQLException if the statement cannot be created
	 */
	public Statement createStatement() throws SQLException{
		Statement stmt = null;
	
			stmt = connection.createStatement();
		return stmt;
		
		
	}

	/**
	 * add the field to the row
	 * @param row the row
	 * @param type the type of field
	 * @param rs the result set
	 * @param index the index of the row for the field
	 * @throws SQLException when the field cannot he added to a row
	 */
	private void addFieldToRow(ArrayList row, int type,ResultSet rs, int index) throws SQLException
	{
		Object object = null;
		switch(type)
		{
			case Types.INTEGER:
				object = new Integer(rs.getInt(index+1));
				break;
			case Types.DATE:
				object = rs.getDate(index+1);
				break;
			case Types.VARCHAR:case Types.CHAR:
				object = rs.getString(index+1);
				break;
			case Types.TIMESTAMP:
				object = rs.getTimestamp(index+1);
				break;
		}
		row.add(index, object);
	}
	
};